var assert = require('assert');
var rgb2hsv = require('../lib/rgb2hsv');
var hsv2rgb = require('../lib/hsv2rgb');

var tests = [
  [[0, 0, 0], [0, 0, 0], '#000000'],
  [[255, 255, 255], [0, 0, 100], '#ffffff'],
  [[255, 0, 0], [0, 100, 100], '#ff0000'],
  [[0, 255, 0], [120, 100, 100], '#00ff00'],
  [[0, 0, 255], [240, 100, 100], '#0000ff'],
  [[255, 255, 0], [60, 100, 100], '#ffff00'],
  [[255, 0, 255], [300, 100, 100], '#00ffff'],
  [[192, 192, 192], [0, 0, 75], '#ff00ff'],
  [[128, 128, 128], [0, 0, 50], '#c0c0c0'],
  [[128, 0, 0], [0, 100, 50], '#808080'],
  [[128, 128, 0], [60, 100, 50], '#800000'],
  [[0, 128, 0], [120, 100, 50], '#808000'],
  [[128, 0, 128], [300, 100, 50], '#800080'],
  [[0, 128, 128], [180, 100, 50], '#008080'],
  [[0, 0, 128], [240, 100, 50], '#008080']
];

describe('Color', function() {
  describe('rgb2hsv', function() {
    tests.forEach(function(test, i) {
      it('should convert #' + (i+1), function() {
        assert.deepEqual(rgb2hsv({
          r: test[0][0],
          g: test[0][1],
          b: test[0][2]
        }), {
          h: test[1][0],
          s: test[1][1],
          v: test[1][2]
        });
      });
    });
  });

  describe('hsv2rgb', function() {
    tests.forEach(function(test, i) {
      it('should convert #' + (i+1), function() {
        assert.deepEqual(hsv2rgb({
          h: test[1][0],
          s: test[1][1],
          v: test[1][2]
        }), {
          r: test[0][0],
          g: test[0][1],
          b: test[0][2]
        });
      });
    });
  });
});